@@ -46,7 +46,8 @@ typedef struct foreach_callback_info_udata_t {
 #define INET6_ADDRSTRLEN 46
 #define INET_PORT 5
 #define IP_PORT_SEPARATOR_LEN 1
-#define IP_PORT_MAX_LEN INET6_ADDRSTRLEN + INET_PORT + IP_PORT_SEPARATOR_LEN
+// Add 2 to the length to facilitate [] around the edges
+#define IP_PORT_MAX_LEN INET6_ADDRSTRLEN + INET_PORT + IP_PORT_SEPARATOR_LEN + 2
 
 /**
  *******************************************************************************************************
@@ -141,10 +142,24 @@ static bool AerospikeClient_Info_each(as_error * err, const as_node * node, cons
 							break;
 						}
 						char ip_port[IP_PORT_MAX_LEN];
+						// If the address is longer than the max length of an ipv6 address, raise an error and exit
+						if (strnlen(host_addr, INET6_ADDRSTRLEN) >= INET6_ADDRSTRLEN) {
+							as_error_update(&udata_ptr->error, AEROSPIKE_ERR_PARAM, "Host address is too long");
+							if (py_res) {
+								Py_DECREF(py_res);
+							}
+							goto CLEANUP;
+						}
 						sprintf(ip_port, "%s:%d", host_addr, port);
 						if ( !strcmp(ip_port, addr->name) ) {
 							PyObject * py_nodes = (PyObject *) udata_ptr->udata_p;
 							PyDict_SetItemString(py_nodes, node->name, py_res);
+						} else {
+							sprintf(ip_port, "[%s]:%d", host_addr, port);
+							if ( !strcmp(ip_port, addr->name) ) {
+								PyObject * py_nodes = (PyObject *) udata_ptr->udata_p;
+								PyDict_SetItemString(py_nodes, node->name, py_res);
+							}
 						}
 					}
 
@@ -165,7 +180,6 @@ static bool AerospikeClient_Info_each(as_error * err, const as_node * node, cons
 	Py_DECREF(py_res);
 
 CLEANUP:
-	PyGILState_Release(gil_state);
 	if (py_ustr) {
 		Py_DECREF(py_ustr);
 	}
@@ -175,6 +189,7 @@ CLEANUP:
 		PyObject *exception_type = raise_exception(&udata_ptr->error);
 		PyErr_SetObject(exception_type, py_err);
 		Py_DECREF(py_err);
+		PyGILState_Release(gil_state);
 		return NULL;
 	}
 	if (err->code != AEROSPIKE_OK) {
@@ -183,8 +198,11 @@ CLEANUP:
 		PyObject *exception_type = raise_exception(err);
 		PyErr_SetObject(exception_type, py_err);
 		Py_DECREF(py_err);
+		PyGILState_Release(gil_state);
 		return NULL;
 	}
+
+	PyGILState_Release(gil_state);
 	return true;
 }
 
