@@ -46,7 +46,8 @@
 	as_policy_operate operate_policy;\
 	as_policy_operate *operate_policy_p = NULL;\
 	as_key key;\
-	char* bin = NULL;
+	bool key_created = false;\
+	char* bin = NULL;\
 
 #define CHECK_CONNECTED_AND_CDT_SUPPORT()\
 	if (!self || !self->as) {\
@@ -71,6 +72,8 @@
 	}\
 	if (pyobject_to_key(&err, py_key, &key) != AEROSPIKE_OK) {\
 		goto CLEANUP;\
+	} else {\
+		key_created = true;\
 	}\
 	if (py_meta) {\
 		if (check_for_meta(py_meta, &ops, &err) != AEROSPIKE_OK) {\
@@ -87,6 +90,9 @@
 	Py_END_ALLOW_THREADS
 
 #define EXCEPTION_ON_ERROR()\
+	if (key_created) {\
+		as_key_destroy(&key);\
+	}\
 	if (err.code != AEROSPIKE_OK) {\
 		PyObject * py_err = NULL;\
 		error_to_pyobject(&err, &py_err);\
@@ -143,6 +149,8 @@ PyObject * AerospikeClient_ListAppend(AerospikeClient * self, PyObject * args, P
 	BASE_VARIABLES
 
 	PyObject * py_append_val = NULL;
+	as_operations ops;
+	as_operations_inita(&ops, 1);
 
 	// Python Function Keyword Arguments
 	static char * kwlist[] = {"key", "bin", "val", "meta", "policy", NULL};
@@ -153,9 +161,6 @@ PyObject * AerospikeClient_ListAppend(AerospikeClient * self, PyObject * args, P
 
 	CHECK_CONNECTED_AND_CDT_SUPPORT();
 
-	as_operations ops;
-	as_operations_inita(&ops, 1);
-
 	as_static_pool static_pool;
 	memset(&static_pool, 0, sizeof(static_pool));
 
@@ -195,6 +200,8 @@ PyObject * AerospikeClient_ListExtend(AerospikeClient * self, PyObject * args, P
 	BASE_VARIABLES
 
 	PyObject * py_append_val = NULL;
+	as_operations ops;
+	as_operations_inita(&ops, 1);
 
 	// Python Function Keyword Arguments
 	static char * kwlist[] = {"key", "bin", "items", "meta", "policy", NULL};
@@ -205,9 +212,6 @@ PyObject * AerospikeClient_ListExtend(AerospikeClient * self, PyObject * args, P
 
 	CHECK_CONNECTED_AND_CDT_SUPPORT();
 
-	as_operations ops;
-	as_operations_inita(&ops, 1);
-
 	as_static_pool static_pool;
 	memset(&static_pool, 0, sizeof(static_pool));
 
@@ -254,7 +258,8 @@ PyObject * AerospikeClient_ListInsert(AerospikeClient * self, PyObject * args, P
 
 	PyObject * py_insert_val = NULL;
 	uint64_t index;
-
+	as_operations ops;
+	as_operations_inita(&ops, 1);
 	// Python Function Keyword Arguments
 	static char * kwlist[] = {"key", "bin", "index", "val", "meta", "policy", NULL};
 	if (PyArg_ParseTupleAndKeywords(args, kwds, "OOlO|OO:list_insert", kwlist,
@@ -264,9 +269,6 @@ PyObject * AerospikeClient_ListInsert(AerospikeClient * self, PyObject * args, P
 
 	CHECK_CONNECTED_AND_CDT_SUPPORT();
 
-	as_operations ops;
-	as_operations_inita(&ops, 1);
-
 	as_static_pool static_pool;
 	memset(&static_pool, 0, sizeof(static_pool));
 
@@ -309,7 +311,8 @@ PyObject * AerospikeClient_ListInsertItems(AerospikeClient * self, PyObject * ar
 
 	PyObject * py_insert_val = NULL;
 	uint64_t index;
-
+	as_operations ops;
+	as_operations_inita(&ops, 1);
 	// Python Function Keyword Arguments
 	static char * kwlist[] = {"key", "bin", "index", "items", "meta", "policy", NULL};
 	if (PyArg_ParseTupleAndKeywords(args, kwds, "OOlO|OO:list_insert_items", kwlist,
@@ -319,9 +322,6 @@ PyObject * AerospikeClient_ListInsertItems(AerospikeClient * self, PyObject * ar
 
 	CHECK_CONNECTED_AND_CDT_SUPPORT();
 
-	as_operations ops;
-	as_operations_inita(&ops, 1);
-
 	as_static_pool static_pool;
 	memset(&static_pool, 0, sizeof(static_pool));
 
@@ -369,6 +369,8 @@ PyObject * AerospikeClient_ListSize(AerospikeClient * self, PyObject * args, PyO
 
 	int64_t list_size = 0;
 	as_record *rec = NULL;
+	as_operations ops;
+	as_operations_inita(&ops, 1);
 
 	// Python Function Keyword Arguments
 	static char * kwlist[] = {"key", "bin", "meta", "policy", NULL};
@@ -379,9 +381,6 @@ PyObject * AerospikeClient_ListSize(AerospikeClient * self, PyObject * args, PyO
 
 	CHECK_CONNECTED_AND_CDT_SUPPORT();
 
-	as_operations ops;
-	as_operations_inita(&ops, 1);
-
 	POLICY_KEY_META_BIN();
 
 	as_operations_add_list_size(&ops, bin);
@@ -428,7 +427,8 @@ PyObject * AerospikeClient_ListPop(AerospikeClient * self, PyObject * args, PyOb
 
 	uint64_t index;
 	as_record *rec = NULL;
-
+	as_operations ops;
+	as_operations_inita(&ops, 1);
 	// Python Function Keyword Arguments
 	static char * kwlist[] = {"key", "bin", "index", "meta", "policy", NULL};
 	if (PyArg_ParseTupleAndKeywords(args, kwds, "OOl|OO:list_pop", kwlist,
@@ -438,9 +438,6 @@ PyObject * AerospikeClient_ListPop(AerospikeClient * self, PyObject * args, PyOb
 
 	CHECK_CONNECTED_AND_CDT_SUPPORT();
 
-	as_operations ops;
-	as_operations_inita(&ops, 1);
-
 	POLICY_KEY_META_BIN();
 
 	as_operations_add_list_pop(&ops, bin, index);
@@ -486,7 +483,8 @@ PyObject * AerospikeClient_ListPopRange(AerospikeClient * self, PyObject * args,
 	uint64_t index;
 	uint64_t count = -1;
 	as_record *rec = NULL;
-
+	as_operations ops;
+	as_operations_inita(&ops, 1);
 	// Python Function Keyword Arguments
 	static char * kwlist[] = {"key", "bin", "index", "count", "meta", "policy", NULL};
 	if (PyArg_ParseTupleAndKeywords(args, kwds, "OOll|OO:list_pop_range", kwlist,
@@ -501,9 +499,6 @@ PyObject * AerospikeClient_ListPopRange(AerospikeClient * self, PyObject * args,
 
 	CHECK_CONNECTED_AND_CDT_SUPPORT();
 
-	as_operations ops;
-	as_operations_inita(&ops, 1);
-
 	POLICY_KEY_META_BIN();
 
 	as_operations_add_list_pop_range(&ops, bin, index, count);
@@ -545,6 +540,8 @@ PyObject * AerospikeClient_ListRemove(AerospikeClient * self, PyObject * args, P
 	BASE_VARIABLES
 
 	uint64_t index;
+	as_operations ops;
+	as_operations_inita(&ops, 1);
 
 	// Python Function Keyword Arguments
 	static char * kwlist[] = {"key", "bin", "index", "meta", "policy", NULL};
@@ -555,9 +552,6 @@ PyObject * AerospikeClient_ListRemove(AerospikeClient * self, PyObject * args, P
 
 	CHECK_CONNECTED_AND_CDT_SUPPORT();
 
-	as_operations ops;
-	as_operations_inita(&ops, 1);
-
 	POLICY_KEY_META_BIN();
 
 	as_operations_add_list_remove(&ops, bin, index);
@@ -590,6 +584,8 @@ PyObject * AerospikeClient_ListRemoveRange(AerospikeClient * self, PyObject * ar
 
 	uint64_t index;
 	uint64_t count = -1;
+	as_operations ops;
+	as_operations_inita(&ops, 1);
 
 	// Python Function Keyword Arguments
 	static char * kwlist[] = {"key", "bin", "index", "count", "meta", "policy", NULL};
@@ -600,9 +596,6 @@ PyObject * AerospikeClient_ListRemoveRange(AerospikeClient * self, PyObject * ar
 
 	CHECK_CONNECTED_AND_CDT_SUPPORT();
 
-	as_operations ops;
-	as_operations_inita(&ops, 1);
-
 	POLICY_KEY_META_BIN();
 
 	as_operations_add_list_remove_range(&ops, bin, index, count);
@@ -632,7 +625,8 @@ CLEANUP:
 PyObject * AerospikeClient_ListClear(AerospikeClient * self, PyObject * args, PyObject * kwds)
 {
 	BASE_VARIABLES
-
+	as_operations ops;
+	as_operations_inita(&ops, 1);
 	// Python Function Keyword Arguments
 	static char * kwlist[] = {"key", "bin", "meta", "policy", NULL};
 	if (PyArg_ParseTupleAndKeywords(args, kwds, "OO|OO:list_clear", kwlist,
@@ -642,9 +636,6 @@ PyObject * AerospikeClient_ListClear(AerospikeClient * self, PyObject * args, Py
 
 	CHECK_CONNECTED_AND_CDT_SUPPORT();
 
-	as_operations ops;
-	as_operations_inita(&ops, 1);
-
 	POLICY_KEY_META_BIN();
 
 	as_operations_add_list_clear(&ops, bin);
@@ -677,7 +668,8 @@ PyObject * AerospikeClient_ListSet(AerospikeClient * self, PyObject * args, PyOb
 
 	PyObject * py_val = NULL;
 	uint64_t index;
-
+	as_operations ops;
+	as_operations_inita(&ops, 1);
 	// Python Function Keyword Arguments
 	static char * kwlist[] = {"key", "bin", "index", "val", "meta", "policy", NULL};
 	if (PyArg_ParseTupleAndKeywords(args, kwds, "OOlO|OO:list_set", kwlist,
@@ -687,9 +679,6 @@ PyObject * AerospikeClient_ListSet(AerospikeClient * self, PyObject * args, PyOb
 
 	CHECK_CONNECTED_AND_CDT_SUPPORT();
 
-	as_operations ops;
-	as_operations_inita(&ops, 1);
-
 	as_static_pool static_pool;
 	memset(&static_pool, 0, sizeof(static_pool));
 
@@ -731,7 +720,8 @@ PyObject * AerospikeClient_ListGet(AerospikeClient * self, PyObject * args, PyOb
 
 	uint64_t index;
 	as_record *rec = NULL;
-
+	as_operations ops;
+	as_operations_inita(&ops, 1);
 	// Python Function Keyword Arguments
 	static char * kwlist[] = {"key", "bin", "index", "meta", "policy", NULL};
 	if (PyArg_ParseTupleAndKeywords(args, kwds, "OOl|OO:list_get", kwlist,
@@ -741,9 +731,6 @@ PyObject * AerospikeClient_ListGet(AerospikeClient * self, PyObject * args, PyOb
 
 	CHECK_CONNECTED_AND_CDT_SUPPORT();
 
-	as_operations ops;
-	as_operations_inita(&ops, 1);
-
 	POLICY_KEY_META_BIN();
 
 	as_operations_add_list_get(&ops, bin, index);
@@ -791,7 +778,8 @@ PyObject * AerospikeClient_ListGetRange(AerospikeClient * self, PyObject * args,
 	uint64_t index;
 	uint64_t count;
 	as_record *rec = NULL;
-
+	as_operations ops;
+	as_operations_inita(&ops, 1);
 	// Python Function Keyword Arguments
 	static char * kwlist[] = {"key", "bin", "index", "count", "meta", "policy", NULL};
 
@@ -803,9 +791,6 @@ PyObject * AerospikeClient_ListGetRange(AerospikeClient * self, PyObject * args,
 
 	CHECK_CONNECTED_AND_CDT_SUPPORT();
 
-	as_operations ops;
-	as_operations_inita(&ops, 1);
-
 	POLICY_KEY_META_BIN();
 
 	as_operations_add_list_get_range(&ops, bin, index, count);
@@ -854,7 +839,8 @@ PyObject * AerospikeClient_ListTrim(AerospikeClient * self, PyObject * args, PyO
 	uint64_t index;
 	uint64_t count;
 	as_record *rec = NULL;
-
+	as_operations ops;
+	as_operations_inita(&ops, 1);
 	// Python Function Keyword Arguments
 	static char * kwlist[] = {"key", "bin", "index", "count", "meta", "policy", NULL};
 
@@ -866,9 +852,6 @@ PyObject * AerospikeClient_ListTrim(AerospikeClient * self, PyObject * args, PyO
 
 	CHECK_CONNECTED_AND_CDT_SUPPORT();
 
-	as_operations ops;
-	as_operations_inita(&ops, 1);
-
 	POLICY_KEY_META_BIN();
 
 	as_operations_add_list_trim(&ops, bin, index, count);
