@@ -39,6 +39,7 @@
 	PyObject * py_key = NULL;\
 	PyObject * py_bin = NULL;\
 	char* bin = NULL;\
+	bool key_created = false;\
 	as_key key;\
 
 #define CHECK_CONNECTED()\
@@ -60,6 +61,8 @@
 	}\
 	if (pyobject_to_key(&err, py_key, &key) != AEROSPIKE_OK) {\
 		goto CLEANUP;\
+	} else {\
+		key_created = true;\
 	}\
 	if (py_meta) {\
 		if (check_for_meta(py_meta, &ops, &err) != AEROSPIKE_OK) {\
@@ -76,7 +79,9 @@
 	}\
 	if (pyobject_to_key(&err, py_key, &key) != AEROSPIKE_OK) {\
 		goto CLEANUP;\
-	}
+	} else {\
+		key_created = true;\
+	}\
 
 #define SETUP_MAP_POLICY()\
 	if (py_mapPolicy) {\
@@ -102,7 +107,9 @@
 #define CLEANUP_AND_EXCEPTION_ON_ERROR(__err)\
 	as_operations_destroy(&ops);\
 	as_record_destroy(rec);\
-	as_key_destroy(&key);\
+	if (key_created) {\
+		as_key_destroy(&key);\
+	}\
 	if (__err.code != AEROSPIKE_OK) {\
 		PyObject * py_err = NULL;\
 		error_to_pyobject(&__err, &py_err);\
@@ -119,12 +126,14 @@ PyObject * AerospikeClient_MapSetPolicy(AerospikeClient * self, PyObject * args,
 	PyObject * py_mapPolicy = NULL;
 	as_map_policy map_policy;
 	as_record *rec = NULL;
+	bool error_occured = false;
 
 	CHECK_CONNECTED();
 
 	static char * kwlist[] = {"key", "bin", "map_policy", NULL};
 	if (PyArg_ParseTupleAndKeywords(args, kwds, "OOO:map_set_policy", kwlist,
 				&py_key, &py_bin, &py_mapPolicy) == false) {
+		error_occured = true;
 		goto CLEANUP;
 	}
 
@@ -140,6 +149,9 @@ PyObject * AerospikeClient_MapSetPolicy(AerospikeClient * self, PyObject * args,
 CLEANUP:
 	CLEANUP_AND_EXCEPTION_ON_ERROR(err);
 
+	if (error_occured) {
+		return NULL;
+	}
 	return PyLong_FromLong(0);
 }
 
@@ -148,6 +160,7 @@ PyObject * AerospikeClient_MapPut(AerospikeClient * self, PyObject * args, PyObj
 	BASE_VARIABLES
 
 	as_static_pool static_pool;
+	memset(&static_pool, 0, sizeof(static_pool));
 
 	PyObject * py_mapKey = NULL;
 	PyObject * py_mapValue = NULL;
@@ -161,12 +174,14 @@ PyObject * AerospikeClient_MapPut(AerospikeClient * self, PyObject * args, PyObj
 	as_record *rec = NULL;
 	as_policy_operate operate_policy;
 	as_policy_operate *operate_policy_p = NULL;
+	bool error_occured = false;
 
 	CHECK_CONNECTED();
 
 	static char * kwlist[] = {"key", "bin", "map_key", "val", "map_policy", "meta", "policy", NULL};
 	if (PyArg_ParseTupleAndKeywords(args, kwds, "OOOO|OOO:map_put", kwlist,
 				&py_key, &py_bin, &py_mapKey, &py_mapValue, &py_mapPolicy, &py_meta, &py_policy) == false) {
+		error_occured = true;
 		goto CLEANUP;
 	}
 
@@ -188,6 +203,9 @@ PyObject * AerospikeClient_MapPut(AerospikeClient * self, PyObject * args, PyObj
 CLEANUP:
 	CLEANUP_AND_EXCEPTION_ON_ERROR(err);
 
+	if (error_occured) {
+		return NULL;
+	}
 	return PyLong_FromLong(0);
 }
 
@@ -196,6 +214,7 @@ PyObject * AerospikeClient_MapPutItems(AerospikeClient * self, PyObject * args,
 	BASE_VARIABLES
 
 	as_static_pool static_pool;
+	memset(&static_pool, 0, sizeof(static_pool));
 
 	PyObject * py_items = NULL;
 	PyObject * py_policy = NULL;
@@ -207,12 +226,14 @@ PyObject * AerospikeClient_MapPutItems(AerospikeClient * self, PyObject * args,
 	as_map * put_items = NULL;
 	as_policy_operate operate_policy;
 	as_policy_operate *operate_policy_p = NULL;
+	bool error_occured = false;
 
 	CHECK_CONNECTED();
 
 	static char * kwlist[] = {"key", "bin", "items", "map_policy", "meta", "policy", NULL};
 	if (PyArg_ParseTupleAndKeywords(args, kwds, "OOO|OOO:map_put_items", kwlist,
 				&py_key, &py_bin, &py_items, &py_mapPolicy, &py_meta, &py_policy) == false) {
+		error_occured = true;
 		goto CLEANUP;
 	}
 
@@ -229,7 +250,9 @@ PyObject * AerospikeClient_MapPutItems(AerospikeClient * self, PyObject * args,
 
 CLEANUP:
 	CLEANUP_AND_EXCEPTION_ON_ERROR(err);
-
+	if (error_occured) {
+		return NULL;
+	}
 	return PyLong_FromLong(0);
 }
 
@@ -237,6 +260,7 @@ PyObject * AerospikeClient_MapIncrement(AerospikeClient * self, PyObject * args,
 	BASE_VARIABLES
 
 	as_static_pool pool;
+	memset(&pool, 0, sizeof(pool));
 
 	PyObject * py_mapKey = NULL;
 	PyObject * py_incr = NULL;
@@ -250,12 +274,14 @@ PyObject * AerospikeClient_MapIncrement(AerospikeClient * self, PyObject * args,
 	as_map_policy_init(&map_policy);
 	as_policy_operate operate_policy;
 	as_policy_operate *operate_policy_p = NULL;
+	bool error_occured = false;
 
 	CHECK_CONNECTED();
 
 	static char * kwlist[] = {"key", "bin", "map_key", "incr", "map_policy", "meta", "policy", NULL};
 	if (PyArg_ParseTupleAndKeywords(args, kwds, "OOOO|OOO:map_increment", kwlist,
 			&py_key, &py_bin, &py_mapKey, &py_incr, &py_mapPolicy, &py_meta, &py_policy) == false) {
+		error_occured = true;
 		goto CLEANUP;
 	}
 
@@ -276,6 +302,9 @@ PyObject * AerospikeClient_MapIncrement(AerospikeClient * self, PyObject * args,
 CLEANUP:
 	CLEANUP_AND_EXCEPTION_ON_ERROR(err);
 
+	if (error_occured) {
+		return NULL;
+	}
 	return PyLong_FromLong(0);
 }
 
@@ -283,7 +312,7 @@ PyObject * AerospikeClient_MapDecrement(AerospikeClient * self, PyObject * args,
 	BASE_VARIABLES
 
 	as_static_pool pool;
-
+	memset(&pool, 0, sizeof(pool));
 	PyObject * py_mapKey = NULL;
 	PyObject * py_decr = NULL;
 	PyObject * py_meta = NULL;
@@ -296,12 +325,14 @@ PyObject * AerospikeClient_MapDecrement(AerospikeClient * self, PyObject * args,
 	as_map_policy_init(&map_policy);
 	as_policy_operate operate_policy;
 	as_policy_operate *operate_policy_p = NULL;
+	bool error_occured = false;
 
 	CHECK_CONNECTED();
 
 	static char * kwlist[] = {"key", "bin", "map_key", "decr", "map_policy", "meta", "policy", NULL};
 	if (PyArg_ParseTupleAndKeywords(args, kwds, "OOOO|OOO:map_decrement", kwlist,
 			&py_key, &py_bin, &py_mapKey, &py_decr, &py_meta, &py_policy) == false) {
+		error_occured = true;
 		goto CLEANUP;
 	}
 
@@ -322,6 +353,9 @@ PyObject * AerospikeClient_MapDecrement(AerospikeClient * self, PyObject * args,
 CLEANUP:
 	CLEANUP_AND_EXCEPTION_ON_ERROR(err);
 
+	if (error_occured) {
+		return NULL;
+	}
 	return PyLong_FromLong(0);
 }
 
@@ -334,12 +368,14 @@ PyObject * AerospikeClient_MapSize(AerospikeClient * self, PyObject * args, PyOb
 	int64_t size = 0;
 	as_policy_operate operate_policy;
 	as_policy_operate *operate_policy_p = NULL;
+	bool error_occured = false;
 
 	CHECK_CONNECTED();
 
 	static char * kwlist[] = {"key", "bin", "meta", "policy", NULL};
 	if (PyArg_ParseTupleAndKeywords(args, kwds, "OO|OO:map_size", kwlist,
 				&py_key, &py_bin, &py_meta, &py_policy) == false) {
+		error_occured = true;
 		goto CLEANUP;
 	}
 
@@ -348,13 +384,16 @@ PyObject * AerospikeClient_MapSize(AerospikeClient * self, PyObject * args, PyOb
 	as_operations_add_map_size(&ops, bin);
 	DO_OPERATION();
 
-	if (rec) {
+	if (rec && as_val_type(rec->bins.entries[0].valuep) != AS_NIL) {
 		size = rec->bins.entries[0].valuep->integer.value;
 	}
 
 CLEANUP:
 	CLEANUP_AND_EXCEPTION_ON_ERROR(err);
 
+	if (error_occured) {
+		return NULL;
+	}
 	return PyLong_FromLong(size);
 }
 
@@ -365,7 +404,7 @@ PyObject * AerospikeClient_MapClear(AerospikeClient * self, PyObject * args, PyO
 	PyObject * py_policy = NULL;
 	as_policy_operate operate_policy;
 	as_policy_operate *operate_policy_p = NULL;
-
+	bool error_occured = false;
 	as_record *rec = NULL;
 
 	CHECK_CONNECTED();
@@ -373,6 +412,7 @@ PyObject * AerospikeClient_MapClear(AerospikeClient * self, PyObject * args, PyO
 	static char * kwlist[] = {"key", "bin", "meta", "policy", NULL};
 	if (PyArg_ParseTupleAndKeywords(args, kwds, "OO|OO:map_clear", kwlist,
 				&py_key, &py_bin, &py_meta, &py_policy) == false) {
+		error_occured = true;
 		goto CLEANUP;
 	}
 
@@ -383,7 +423,9 @@ PyObject * AerospikeClient_MapClear(AerospikeClient * self, PyObject * args, PyO
 
 CLEANUP:
 	CLEANUP_AND_EXCEPTION_ON_ERROR(err);
-
+	if (error_occured) {
+		return NULL;
+	}
 	return PyLong_FromLong(0);
 }
 
@@ -392,6 +434,7 @@ PyObject * AerospikeClient_MapRemoveByKey(AerospikeClient * self, PyObject * arg
 	BASE_VARIABLES
 
 	as_static_pool pool;
+	memset(&pool, 0, sizeof(pool));
 
 	PyObject * py_result = NULL;
 	PyObject * py_mapKey = NULL;
@@ -434,6 +477,7 @@ PyObject * AerospikeClient_MapRemoveByKeyList(AerospikeClient * self, PyObject *
 	BASE_VARIABLES
 
 	as_static_pool pool;
+	memset(&pool, 0, sizeof(pool));
 
 	PyObject * py_result = NULL;
 	PyObject * py_list = NULL;
@@ -481,6 +525,7 @@ PyObject * AerospikeClient_MapRemoveByKeyRange(AerospikeClient * self, PyObject
 	BASE_VARIABLES
 
 	as_static_pool pool;
+	memset(&pool, 0, sizeof(pool));
 
 	PyObject * py_mapKey = NULL;
 	PyObject * py_result = NULL;
@@ -529,6 +574,7 @@ PyObject * AerospikeClient_MapRemoveByValue(AerospikeClient * self, PyObject * a
 	BASE_VARIABLES
 
 	as_static_pool pool;
+	memset(&pool, 0, sizeof(pool));
 
 	PyObject * py_mapValue = NULL;
 	PyObject * py_result = NULL;
@@ -572,6 +618,7 @@ PyObject * AerospikeClient_MapRemoveByValueList(AerospikeClient * self, PyObject
 	BASE_VARIABLES
 
 	as_static_pool pool;
+	memset(&pool, 0, sizeof(pool));
 
 	PyObject * py_result = NULL;
 	PyObject * py_list = NULL;
@@ -620,6 +667,7 @@ PyObject * AerospikeClient_MapRemoveByValueRange(AerospikeClient * self, PyObjec
 	BASE_VARIABLES
 
 	as_static_pool pool;
+	memset(&pool, 0, sizeof(pool));
 
 	PyObject * py_mapValue = NULL;
 	PyObject * py_result = NULL;
@@ -806,6 +854,7 @@ PyObject * AerospikeClient_MapGetByKey(AerospikeClient * self, PyObject * args,
 	BASE_VARIABLES
 
 	as_static_pool pool;
+	memset(&pool, 0, sizeof(pool));
 
 	PyObject * py_mapKey = NULL;
 	PyObject * py_result = NULL;
@@ -849,6 +898,7 @@ PyObject * AerospikeClient_MapGetByValue(AerospikeClient * self, PyObject * args
 	BASE_VARIABLES
 
 	as_static_pool pool;
+	memset(&pool, 0, sizeof(pool));
 
 	PyObject * py_mapValue = NULL;
 	PyObject * py_result = NULL;
@@ -891,6 +941,7 @@ PyObject * AerospikeClient_MapGetByKeyRange(AerospikeClient * self, PyObject * a
 	BASE_VARIABLES
 
 	as_static_pool pool;
+	memset(&pool, 0, sizeof(pool));
 
 	PyObject * py_mapKey = NULL;
 	PyObject * py_result = NULL;
@@ -939,6 +990,7 @@ PyObject * AerospikeClient_MapGetByValueRange(AerospikeClient * self, PyObject *
 	BASE_VARIABLES
 
 	as_static_pool pool;
+	memset(&pool, 0, sizeof(pool));
 
 	PyObject * py_mapValue = NULL;
 	PyObject * py_result = NULL;
