@@ -555,7 +555,7 @@ as_status pyobject_to_record(AerospikeClient * self, as_error * err, PyObject *
 				}
 
 				if (aerospike_has_geo(self->as)) {
-					ret_val = as_record_set_geojson_str(rec, name, geo_value);
+					ret_val = as_record_set_geojson_strp(rec, name, strdup(geo_value), true);
 				} else {
 					as_bytes *bytes;
 					GET_BYTES_POOL(bytes, static_pool, err);
@@ -1084,6 +1084,7 @@ as_status as_list_of_map_to_py_tuple_list(AerospikeClient * self, as_error * err
 		PyTuple_SetItem(py_tuple, 1, py_value);
 
 		PyList_Append(*py_list, py_tuple);
+		Py_DECREF(py_tuple);
 	}
 
 CLEANUP:
@@ -1281,7 +1282,11 @@ as_status key_to_pyobject(as_error * err, const as_key * key, PyObject ** obj)
 				py_key = PyString_FromString( as_string_get(sval) );
 				if (!py_key){
 					py_key = PyUnicode_DecodeUTF8(as_string_get(sval), as_string_len(sval), NULL);
+					PyErr_Clear();
 				}
+				if (!py_key) {
+					py_key = PyBytes_FromString(as_string_get(sval));
+                }
 				if (!py_key) {
 					as_error_update(err, AEROSPIKE_ERR_CLIENT, "Unknown type for value");
 					return err->code;
